==> mule-extensions-maven-plugin-1.2.4-test-sources.jar: org/mule/extension/maven/ExtensionDescriptorMojoTest.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.extension.maven;

import static java.lang.System.setProperty;
import static java.nio.file.Paths.get;
import static org.apache.commons.io.FileUtils.copyFile;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsInstanceOf.instanceOf;
import static org.hamcrest.core.StringContains.containsString;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.mule.plugin.maven.AbstractMuleMojo.MULE_ARTIFACT;
import static org.mule.plugin.maven.AbstractMuleMojo.MULE_ARTIFACT_JSON;
import static org.powermock.api.mockito.PowerMockito.mockStatic;

import org.mule.tck.junit4.rule.SystemProperty;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.nio.file.Path;
import java.util.Properties;

import org.apache.commons.io.IOUtils;
import org.apache.maven.model.Build;
import org.apache.maven.project.MavenProject;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ExtensionDescriptorMojo.class})
public class ExtensionDescriptorMojoTest {

  private static final String MULE_ARTIFACT_JSON_LOCATION = get("META-INF", MULE_ARTIFACT, MULE_ARTIFACT_JSON).toString();
  private static final String MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION = "without-minmuleversion-" + MULE_ARTIFACT_JSON;
  private static final String SKIP_MULE_VERSION_SUFFIX_VALIDATION_PROPERTY = "mule.extensions.disableMuleVersionSuffixValidation";
  public static final String MULE_VERSION_PROPERTY = "mule.version";
  public static final String MIN_MULE_VERSION_PROPERTY = "min.mule.version";
  public static final String MULE_VERSION = "4.2.0-SNAPSHOT";
  public static final String MULE_ARTIFACT_JSON_MIN_MULE_VERSION = "4.2.0-SNAPSHOT";
  public static final String VALID_MIN_MULE_VERSION = "4.2.1";
  public static final String INVALID_MIN_MULE_VERSION = "4.1.5";
  public static final String MIN_MULE_VERSION_WITH_INVALID_SUFFIX = "4.2.0-XRANDOMSUFFIX";
  public static final String PROJECT_VERSION = "1.1.0-SNAPSHOT";

  private MavenProject projectMock;
  private Build buildMock;
  private Properties props;

  private ExtensionDescriptorMojo mojo;

  @Rule
  public TemporaryFolder projectBaseFolder = new TemporaryFolder();

  @Rule
  public TemporaryFolder buildFolderFolder = new TemporaryFolder();

  @Rule
  public ExpectedException expectedException = ExpectedException.none();

  @Rule
  public SystemProperty systemProperty = new SystemProperty(SKIP_MULE_VERSION_SUFFIX_VALIDATION_PROPERTY, "false");

  @Before
  public void setUp() throws Exception {
    mojo = new ExtensionDescriptorMojo();

    projectMock = mock(MavenProject.class);
    buildMock = mock(Build.class);
    props = mock(Properties.class);

    mojo.project = projectMock;

    when(projectMock.getBuild()).thenReturn(buildMock);
    when(projectMock.getProperties()).thenReturn(props);
    when(projectMock.getVersion()).thenReturn(PROJECT_VERSION);
    when(buildMock.getOutputDirectory()).thenReturn(buildFolderFolder.getRoot().getAbsolutePath());
    when(props.get(MULE_VERSION_PROPERTY)).thenReturn(MULE_VERSION);
  }

  @Test
  public void testMuleArtifactGetsProperlyGenerated() throws Exception {
    executeMojo(MULE_ARTIFACT_JSON);
    checkResultingMinMuleVersion(MULE_ARTIFACT_JSON_MIN_MULE_VERSION);
  }

  @Test
  public void testMuleArtifactGetsProperlyGeneratedWhenMinMuleVersionIsSet() throws Exception {
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(VALID_MIN_MULE_VERSION);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
    checkResultingMinMuleVersion(VALID_MIN_MULE_VERSION);
  }

  @Test
  public void testMuleArtifactIsNotProperlyGeneratedWhenInvalidMinMuleVersionIsSet() throws Exception {
    expectedException.expect(instanceOf(IllegalArgumentException.class));
    expectedException.expectMessage("has to be at least equal or higher than the mule version");
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(INVALID_MIN_MULE_VERSION);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
  }

  @Test
  public void minMuleVersionHasInvalidSuffix() throws Exception {
    expectedException.expect(instanceOf(IllegalArgumentException.class));
    expectedException.expectMessage("minMuleVersion cannot have suffix different from SNAPSHOT");
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(MIN_MULE_VERSION_WITH_INVALID_SUFFIX);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
  }

  @Test
  public void minMuleVersionHasInvalidSuffixButValidationIsSkipped() throws Exception {
    setProperty(SKIP_MULE_VERSION_SUFFIX_VALIDATION_PROPERTY, "true");
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(MIN_MULE_VERSION_WITH_INVALID_SUFFIX);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
    checkResultingMinMuleVersion(MIN_MULE_VERSION_WITH_INVALID_SUFFIX);
  }

  private void checkResultingMinMuleVersion(String muleArtifactJsonMinMuleVersion) throws Exception {
    String buildFolderPath = buildFolderFolder.getRoot().getAbsolutePath();
    Path muleArtifactAbsolutePath = get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION);
    InputStream resultingArtifactJson = new FileInputStream(new File(muleArtifactAbsolutePath.toAbsolutePath().toString()));
    assertThat(IOUtils.toString(resultingArtifactJson),
               containsString("\"minMuleVersion\": \"" + muleArtifactJsonMinMuleVersion + "\""));
  }

  private void executeMojo(String pathToMuleArtifactJson) throws Exception {
    String buildFolderPath = buildFolderFolder.getRoot().getAbsolutePath();
    Path muleArtifactAbsolutePath = get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION);

    mockStatic(ExtensionDescriptorMojo.class);
    PowerMockito.when(ExtensionDescriptorMojo.descriptorPathOrFail(anyObject())).thenReturn(muleArtifactAbsolutePath);
    PowerMockito.when(ExtensionDescriptorMojo.autoGeneratedDescriptorPath(anyObject())).thenCallRealMethod();

    copyFile(new File(ExtensionDescriptorMojoTest.class.getClassLoader().getResource(pathToMuleArtifactJson).getFile()),
             get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION).toFile());

    mojo.execute();
  }
}

==> mule-extensions-maven-plugin-1.3.0-test-sources.jar: org/mule/extension/maven/ExtensionDescriptorMojoTest.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.extension.maven;

import static java.nio.file.Paths.get;
import static org.apache.commons.io.FileUtils.copyFile;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsInstanceOf.instanceOf;
import static org.hamcrest.core.StringContains.containsString;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.mule.plugin.maven.AbstractMuleMojo.MULE_ARTIFACT;
import static org.mule.plugin.maven.AbstractMuleMojo.MULE_ARTIFACT_JSON;
import static org.powermock.api.mockito.PowerMockito.mockStatic;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.util.Properties;

import org.apache.commons.io.IOUtils;
import org.apache.maven.model.Build;
import org.apache.maven.project.MavenProject;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.TemporaryFolder;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ExtensionDescriptorMojo.class})
public class ExtensionDescriptorMojoTest {

  private static final String MULE_ARTIFACT_JSON_LOCATION = get("META-INF", MULE_ARTIFACT, MULE_ARTIFACT_JSON).toString();
  private static final String MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION = "without-minmuleversion-" + MULE_ARTIFACT_JSON;
  public static final String MULE_VERSION_PROPERTY = "mule.version";
  public static final String MIN_MULE_VERSION_PROPERTY = "min.mule.version";
  public static final String MULE_VERSION = "4.2.0-SNAPSHOT";
  public static final String MULE_ARTIFACT_JSON_MIN_MULE_VERSION = "4.2.0-SNAPSHOT";
  public static final String VALID_MIN_MULE_VERSION = "4.2.1";
  public static final String INVALID_MIN_MULE_VERSION = "4.1.5";
  public static final String PROJECT_VERSION = "1.1.0-SNAPSHOT";

  private MavenProject projectMock;
  private Build buildMock;
  private Properties props;

  private ExtensionDescriptorMojo mojo;

  @Rule
  public TemporaryFolder projectBaseFolder = new TemporaryFolder();

  @Rule
  public TemporaryFolder buildFolderFolder = new TemporaryFolder();

  @Rule
  public ExpectedException expectedException = ExpectedException.none();

  @Before
  public void setUp() throws Exception {
    mojo = new ExtensionDescriptorMojo();

    projectMock = mock(MavenProject.class);
    buildMock = mock(Build.class);
    props = mock(Properties.class);

    mojo.project = projectMock;

    when(projectMock.getBuild()).thenReturn(buildMock);
    when(projectMock.getProperties()).thenReturn(props);
    when(projectMock.getVersion()).thenReturn(PROJECT_VERSION);
    when(buildMock.getOutputDirectory()).thenReturn(buildFolderFolder.getRoot().getAbsolutePath());
    when(props.get(MULE_VERSION_PROPERTY)).thenReturn(MULE_VERSION);
  }

  @Test
  public void testMuleArtifactGetsProperlyGenerated() throws Exception {
    executeMojo(MULE_ARTIFACT_JSON);
    checkResultingMinMuleVersion(MULE_ARTIFACT_JSON_MIN_MULE_VERSION);
  }

  @Test
  public void testMuleArtifactGetsProperlyGeneratedWhenMinMuleVersionIsSet() throws Exception {
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(VALID_MIN_MULE_VERSION);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
    checkResultingMinMuleVersion(VALID_MIN_MULE_VERSION);
  }

  @Test
  public void testMuleArtifactIsNotProperlyGeneratedWhenInvalidMinMuleVersionIsSet() throws Exception {
    expectedException.expect(instanceOf(IllegalArgumentException.class));
    expectedException.expectMessage("has to be at least equal or higher than the mule version");
    when(props.get(MIN_MULE_VERSION_PROPERTY)).thenReturn(INVALID_MIN_MULE_VERSION);
    executeMojo(MULE_ARTIFACT_JSON_WITHOUT_MIN_MULE_VERSION);
  }


  private void checkResultingMinMuleVersion(String muleArtifactJsonMinMuleVersion) throws Exception {
    String buildFolderPath = buildFolderFolder.getRoot().getAbsolutePath();
    Path muleArtifactAbsolutePath = get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION);
    InputStream resultingArtifactJson = new FileInputStream(new File(muleArtifactAbsolutePath.toAbsolutePath().toString()));
    assertThat(IOUtils.toString(resultingArtifactJson),
               containsString("\"minMuleVersion\": \"" + muleArtifactJsonMinMuleVersion + "\""));
  }

  private void executeMojo(String pathToMuleArtifactJson) throws Exception {
    String buildFolderPath = buildFolderFolder.getRoot().getAbsolutePath();
    Path muleArtifactAbsolutePath = get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION);

    mockStatic(ExtensionDescriptorMojo.class);
    PowerMockito.when(ExtensionDescriptorMojo.descriptorPathOrFail(anyObject())).thenReturn(muleArtifactAbsolutePath);
    PowerMockito.when(ExtensionDescriptorMojo.autoGeneratedDescriptorPath(anyObject())).thenCallRealMethod();

    copyFile(new File(ExtensionDescriptorMojoTest.class.getClassLoader().getResource(pathToMuleArtifactJson).getFile()),
             get(buildFolderPath, MULE_ARTIFACT_JSON_LOCATION).toFile());

    mojo.execute();
  }
}
